/*
 * GeneratedReport.java
 *
 * Created on 9  2001 ., 16:54
 */

package com.fn.repway;


import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 * @version 
 */
public class GeneratedReport implements java.awt.print.Pageable {

    private List pages = new ArrayList();
    private Page lastPage = null;
    private String name;
    private int side = Report.FRONT_SIDE;
    private Map offscreens = new HashMap();
    
    /** Creates new GeneratedReport */
    public GeneratedReport(String name) {
        this.name = name;
    }

    /** Creates GeneratedReport from XML */
    public GeneratedReport(Element tag) throws RepException {
        name = XMLUtils.getAttrib(tag, "name", "");
        for (XMLIterator i = new XMLIterator(tag); i.hasNext(); ) {
            Element subtag = i.next();
            if (subtag != null) {
                if (subtag.getTagName().equals("page"))
                    pages.add(new Page(this, subtag));
                else
                    if (subtag.getTagName().equals("offscreen"))
                        loadOffscreens(subtag);
            }
        }
    }
    
    private void loadOffscreens(Element tag) throws RepException {
        for (XMLIterator i = new XMLIterator(tag); i.hasNext(); ) {
            Element subtag = i.next();
            if (subtag != null) {
                String name = XMLUtils.getAttrib(subtag, "name", "");
                offscreens.put(name,  OffscreensFactory.load(subtag));
            }
        }
    }
    
    public String getName() {
        return name;
    }
    
    public Page addPage(PageFormat format) {
        lastPage = new Page(this, format);
        pages.add(lastPage);
        return lastPage;
    }
    
    public Page getLastPage() {
        return lastPage;
    }
    
    public Page getPage(int no) {
        return (Page)pages.get(no);
    }
    
    public void print() throws Exception {
        java.awt.print.PrinterJob printerJob = java.awt.print.PrinterJob.getPrinterJob();
        printerJob.setJobName(name);
        printerJob.setPageable(this);
        if (printerJob.printDialog()) 
            printerJob.print();
    }
    
    public java.awt.print.Printable getPrintable(int no) throws java.lang.IndexOutOfBoundsException {
        return (Page)pages.get(no);
    }
    
    public java.awt.print.PageFormat getPageFormat(int no) throws java.lang.IndexOutOfBoundsException {
        PageFormat pageFormat = ((Page)pages.get(no)).getPageFormat();
        java.awt.print.Paper paper = new java.awt.print.Paper();
        double w = pageFormat.getWidth() * 72.0/25.4;
        double h = pageFormat.getHeight() * 72.0/25.4;
        paper.setSize(w, h);
        paper.setImageableArea(0.0, 0.0, w, h);
        java.awt.print.PageFormat apf = new java.awt.print.PageFormat();
        apf.setPaper(paper);
        if (pageFormat.getOrient() == PageFormat.LANDSCAPE)
            apf.setOrientation(PageFormat.LANDSCAPE);
        else
            apf.setOrientation(PageFormat.PORTRAIT);
        return apf;
    }
    
    public int getNumberOfPages() {
        return pages.size();
    }
    
    public int getSide() {
        return side;
    }
    
    public void setSide(int side) {
        this.side = side;
    }
    
    public boolean hasFrontSide() {
        for (Iterator i = pages.listIterator(); i.hasNext(); )
            if (((Page)i.next()).hasFrontSide())
                return true;
        return false;
    }
    
    public boolean hasBackSide() {
        for (Iterator i = pages.listIterator(); i.hasNext(); )
            if (((Page)i.next()).hasBackSide())
                return true;
        return false;
    }
    
    public void addOffscreen(String name, OffscreenStorable offscrn) {
        offscreens.put(name, offscrn);
    }
    
    public OffscreenStorable getOffscreen(String name) {
        return (OffscreenStorable)offscreens.get(name);
    }
    
    private void saveOffscreens(Writer out) throws IOException {
        out.write("<offscreen>");
        for (Iterator i = offscreens.entrySet().iterator(); i.hasNext(); ) {
            Map.Entry entry = (Map.Entry)i.next();
            String name = (String)entry.getKey();
            OffscreenStorable value = (OffscreenStorable)entry.getValue();
            value.save(out, name);
        }
        out.write("</offscreen>");
    }
    
    private void saveBody(Writer out) throws IOException {
        out.write("<genrep name=\"");
        out.write(XMLUtils.escape(name));
        out.write("\">");
        saveOffscreens(out);
        for (Iterator i = pages.listIterator(); i.hasNext(); ) {
            Page p = (Page)i.next();
            p.save(out);
        }
        out.write("</genrep>");
    }
    
    public void save(String encoding, Writer out) throws IOException {
        out.write("<?xml version=\"1.0\" encoding=\"" + encoding + "\"?>");
        saveBody(out);
    }

    public void save(Writer out) throws IOException {
        out.write("<?xml version=\"1.0\"?>");
        saveBody(out);
    }
}
