/*
 * Label.java
 *
 * Created on 9  2001 ., 18:14
 */

package com.fn.repway;


import java.util.*;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 * @version 
 */
class Label extends RepElement {

    public static final int ALIGN_LEFT = 1;
    public static final int ALIGN_CENTER = 2;
    public static final int ALIGN_RIGHT = 3;
    
    private static final int FONT_NORMAL = 0;
    private static final int FONT_BOLD = 1;
    private static final int FONT_ITALIC = 2;
    private static final int FONT_BOLDITALIC = 3;
    
    private java.awt.Font font;
    private int align;
    private java.awt.Color color;
    private boolean clip;
    private double rotation;
    private boolean trim;
    
    private List fields = new LinkedList();
    
    /** Creates new Label */
    public Label(Element tag) throws RepException {
        super(tag);
        
        String eType = XMLUtils.getAttrib(tag, "type", (String)null);
        if (eType != null)
            fields.add(FieldFactory.createField(eType, tag));
   
        XMLIterator xmlIterator = new XMLIterator(tag);
        if (xmlIterator.hasNext())
            while (xmlIterator.hasNext()) {
                Element subtag = xmlIterator.next();
                if (subtag.getTagName().equals("field"))
                    fields.add(FieldFactory.createField(XMLUtils.getAttrib(subtag, "type", ""), subtag));
            }

        String fontName = XMLUtils.getAttrib(tag, "font", "Times");
        double fs = XMLUtils.getAttrib(tag, "size", 12.0);
        float fontSize = (float)(fs / 72.0 * 25.4);
        if (height < 0.0)
            height = fontSize * 1.3;
        int fontStyle = parseStyle(XMLUtils.getAttrib(tag, "style", "plain"));
        java.util.Map fontParams = new java.util.HashMap();
        if ((fontStyle & FONT_ITALIC) != 0)
            fontParams.put(java.awt.font.TextAttribute.POSTURE, java.awt.font.TextAttribute.POSTURE_OBLIQUE);
        else
            fontParams.put(java.awt.font.TextAttribute.POSTURE, java.awt.font.TextAttribute.POSTURE_REGULAR);
        if ((fontStyle & FONT_BOLD) != 0)
            fontParams.put(java.awt.font.TextAttribute.WEIGHT, java.awt.font.TextAttribute.WEIGHT_BOLD);
        else
            fontParams.put(java.awt.font.TextAttribute.WEIGHT, java.awt.font.TextAttribute.WEIGHT_REGULAR);
        fontParams.put(java.awt.font.TextAttribute.SIZE, new Float(fontSize));
        fontParams.put(java.awt.font.TextAttribute.FAMILY, fontName);
        //font = new java.awt.Font(fontName, fontStyle, fontSize);
        font = new java.awt.Font(fontParams);
        if (font == null)
            throw new RepException("Invalid font");
        color = XMLUtils.getColorAttrib(tag, "color", "000000");
        align = parseAlign(XMLUtils.getAttrib(tag, "align", "left"));
        clip = XMLUtils.getAttrib(tag, "clip", true);
        rotation = XMLUtils.getAttrib(tag, "rotate", 0.0);
        trim = XMLUtils.getAttrib(tag, "trim", false);
    }

    private int parseStyle(String style) throws RepException {
        String s = style.toLowerCase();
        if (s.equals("plain") || s.equals(""))
            return java.awt.Font.PLAIN;
        else if (s.equals("italic") || s.equals("oblique") || s.equals("i") || s.equals("o"))
            return java.awt.Font.ITALIC;
        else if (s.equals("bold") || s.equals("b"))
            return java.awt.Font.BOLD;
        else if (s.equals("italicbold") || s.equals("italic bold") || s.equals("italic&bold") || 
                s.equals("italic & bold") || s.equals("obliquebold") || s.equals("oblique bold") || 
                s.equals("oblique&bold") || s.equals("oblique & bold") || s.equals("ib") || 
                s.equals("ob") || 
                s.equals("bi") || s.equals("bo"))
            return java.awt.Font.BOLD | java.awt.Font.ITALIC;
        else
            throw new RepException("Invalid font style.");
    }
    
    private int parseAlign(String align) throws RepException {
        String s = align.toLowerCase();
        if (s.equals("left") || s.equals(""))
            return ALIGN_LEFT;
        else if (s.equals("right"))
            return ALIGN_RIGHT;
        else if (s.equals("center"))
            return ALIGN_CENTER;
        else
            throw new RepException("Invalid text aligment.");
    }
    
    public void generate(GenContext ctx, Band band, boolean front) throws RepException {
        StringBuffer sb = new StringBuffer();
        for (Iterator i = fields.listIterator(); i.hasNext(); ) {
            DataField field = (DataField)i.next();
            String s = field.getText(ctx).getAsString();
            if (s != null)
                sb.append(s);
        }
        band.add(front, new CompLabel(left, top, width, height, font, color, align, clip,
            trim ? sb.toString().trim() : sb.toString(), rotation));
    }
    
    public void setDataField(DataField field) {
        fields.clear();
        fields.add(field);
    }
}
