/*
 * Page.java
 *
 * Created on 9  2001 ., 18:27
 */

package com.fn.repway;


import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 * @version 
 */
class Page implements java.awt.print.Printable {

    private PageFormat pageFormat;
    private List frontSide = new LinkedList();
    private List backSide = new LinkedList();
    private GeneratedReport report;

    /** Creates new Page */
    public Page(GeneratedReport report, PageFormat pageFormat) {
        this.report = report;
        this.pageFormat = pageFormat;
    }

    public Page(GeneratedReport report, Element tag) throws RepException {
        this.report = report;
        pageFormat = new PageFormat(XMLUtils.getSubTag(tag, ""));
        loadSide(tag, "front", frontSide);
        loadSide(tag, "back", backSide);
    }
    
    private void loadSide(Element tag, String name, List list) throws RepException {
        Element subtag = null;
        try {
            subtag = XMLUtils.getSubTag(tag, name);
        } catch (Exception e) {
            return;
        }
        if (subtag == null) return;
        for (XMLIterator i = new XMLIterator(subtag); i.hasNext(); ) {
            Element e = i.next();
            list.add(PrimitivesFactory.create(report, e.getTagName(), e));
        }
    }
    
    void add(double x, double y, Band band) {
        for (Iterator i = band.getFrontSide().listIterator(); i.hasNext(); ) {
            Primitive p = (Primitive)i.next();
            p.moveBy(x, y);
            frontSide.add(p);
        }
        for (Iterator i = band.getBackSide().listIterator(); i.hasNext(); ) {
            Primitive p = (Primitive)i.next();
            p.moveBy(x, y);
            backSide.add(p);
        }
    }
    
    public PageFormat getPageFormat() {
        return pageFormat;
    }
    
    public void draw(java.awt.Graphics2D g2) {
        Iterator i;
        if (report.getSide() == Report.FRONT_SIDE)
            i = frontSide.listIterator();
        else
            i = backSide.listIterator();
        while (i.hasNext())
            ((Primitive)i.next()).draw(g2);
    }
    
    public int print(java.awt.Graphics graphics, java.awt.print.PageFormat pageFormat, 
            int pageIndex) throws java.awt.print.PrinterException {
        java.awt.Graphics2D g2 = (java.awt.Graphics2D)graphics;
        java.awt.geom.AffineTransform oldt = g2.getTransform();
        java.awt.geom.AffineTransform newt = new java.awt.geom.AffineTransform(oldt);
        newt.scale(72.0/25.4, 72.0/25.4);
        g2.setTransform(newt);
        draw(g2);
        g2.setTransform(oldt);
        return java.awt.print.Printable.PAGE_EXISTS;
    }
    
    public boolean hasFrontSide() {
        if ((frontSide != null) && (frontSide.size() > 0))
            return true;
        else
            return false;
    }
    
    public boolean hasBackSide() {
        if ((backSide != null) && (backSide.size() > 0))
            return true;
        else
            return false;
    }
    
    private void saveSide(Writer out, String name, List side) throws IOException {
        out.write("<" + name + ">");
        for (Iterator i = side.iterator(); i.hasNext(); ) {
            Primitive p = (Primitive)i.next();
            p.save(out);
        }
        out.write("</" + name + ">");
    }
    
    public void save(Writer out) throws IOException {
        out.write("<page>");
        pageFormat.save(out);
        if (hasFrontSide())
            saveSide(out, "front", frontSide);
        if (hasBackSide())
            saveSide(out, "back", backSide);
        out.write("</page>");
    }
}
