/*
 * Part.java
 *
 * Created on 9  2001 ., 12:53
 */

package com.fn.repway;


import java.util.*;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 * @version 
 */
public class Part {
    private List sections = new LinkedList();
    private Section pageHeader = null;
    private Section pageFooter = null;
    private String name;
    private boolean startNewPage;
    private boolean finishOldPage = false;
    private PageFormat pageFormat = null;
    private Border border = null;
    private Layout layout = null;
    private Report report;

    public Part(Report report, String name) {
        this.report = report;
        this.name = name;
    }

    /** Creates new Part */
    Part(Report report, Element tag) throws RepException {
        this.report = report;
        name = XMLUtils.getAttrib(tag, "name", "");
        startNewPage = XMLUtils.getAttrib(tag, "newpage", true);
        
        for (XMLIterator i = new XMLIterator(tag); i.hasNext(); ) {
            Element subtag = i.next();
            String n = subtag.getTagName();
            if (n.equals("section")) {
                Section s = new Section(report, subtag);
                String st = s.getType();
                if (st.equals("body"))
                    sections.add(s);
                else if (st.equals("pagefooter"))
                    pageFooter = s;
                else if (st.equals("pageheader"))
                    pageHeader = s;
                else
                    throw new RepException("Invalid section type '" + st + "'");
            } else if (n.equals("layout"))
                layout = LayoutFactory.createLayout(subtag);
            else if (n.equals("paper"))
                pageFormat = new PageFormat(subtag);
            else if (n.equals("border"))
                border = new Border(subtag);
            else
                throw new RepException("Invalid part component + '" + n + "'");
        }        
    }
    
    public List getSections() {
        return sections;
    }
    
    public String getName() {
        return name;
    }
        
    public void generate(GenContext ctx) throws RepException {
        ctx.pushPageHeader(pageHeader);
        ctx.pushPageFooter(pageFooter);
        ctx.pushLayout(layout);
        ctx.pushBorder(border);
        ctx.pushPageFormat(pageFormat);
        
        ctx.getLayout().begin(ctx);
        
        if ((ctx.getCurrentPage() == null) || startNewPage)
            ctx.getLayout().startPage(ctx);

        for (Iterator i = sections.listIterator(); i.hasNext(); )
            ((Section)i.next()).generate(ctx);
        
        if (finishOldPage)
            ctx.getLayout().finishPage(ctx);
        
        ctx.getLayout().end(ctx);
        
        ctx.popPageHeader(pageHeader);
        ctx.popPageFooter(pageFooter);
        ctx.popLayout(layout);
        ctx.popBorder(border);
        ctx.popPageFormat(pageFormat);        
    }
    
    public boolean getStartNewPage() {
        return startNewPage;
    }
    
    public void setStartNewPage(boolean newPage) {
        startNewPage = newPage;
    }
    
    public boolean getFinishOldPage() {
        return finishOldPage;
    }
    
    public void setFinishOldPage(boolean finishPage) {
        finishOldPage = finishPage;
    }
    
    public boolean isTwoSided() {
        if (pageHeader != null)
            if (pageHeader.isTwoSided()) return true;
        if (pageFooter != null)
            if (pageFooter.isTwoSided()) return true;
        
        for (Iterator i = sections.listIterator(); i.hasNext(); ) {
            Section s = (Section)i.next();
            if (s.isTwoSided())
                return true;
        }
        
        return false;
    }
    
    public Section getPageHeader() {
        return pageHeader;
    }
    
    public void setPageHeader(Section section) {
        pageHeader = section;
    }
    
    public Section getPageFooter() {
        return pageFooter;
    }
    
    public void setPageFooter(Section section) {
        pageFooter = section;
    }
}
