/*
 * Report.java
 *
 * Created on 9  2001 ., 12:21
 */

package com.fn.repway;


import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringReader;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 * @version 
 */
public class Report {
    private List parts = new LinkedList();    
    private PageFormat pageFormat = null;
    private Border border = null;
    private Section pageHeader = null;
    private Section pageFooter = null;
    private Layout layout = null;
    private String name = "ReportWay";
    private DataLoader dataLoader = null;
    private Map namedSections = new HashMap();
    
    public static final int FRONT_SIDE = 1;
    public static final int BACK_SIDE = 2;
    
    public Report() {
        pageFormat = new PageFormat();
        border = new Border();
        name = "ReportWay";
    }
    
    /** Creates new Report */
    public Report(Reader reader) throws RepException {
        dataLoader = new FileDataLoader();
        load(reader);
    }
    
    public Report(Element tag) throws RepException {
        dataLoader = new FileDataLoader();
        load(tag);
    }
    
    public Report(String report) throws RepException {
        dataLoader = new FileDataLoader();
        load(report);
    }
    
    public Report(java.net.URL report) throws RepException {
        String path = report.toExternalForm();
        int idx = path.lastIndexOf('/');
        if (-1 != idx)
            dataLoader = new URLDataLoader(path.substring(0, idx));
        else
            dataLoader = new URLDataLoader();
        try {
            load(new BufferedReader(new InputStreamReader(report.openStream())));
        } catch (java.io.IOException e) {
            throw new RepException("Error reading URL: " + e.getMessage());
        }
    }
    
    public void load(Reader reader) throws RepException {
        load(XMLUtils.parse(reader));
    }
    
    public void load(String data) throws RepException {
        load(XMLUtils.parse(new StringReader(data)));
    }
    
    public void load(Element tag) throws RepException {
        name = XMLUtils.getAttrib(tag, "name", "ReportWay");
        pageFormat = new PageFormat(tag);
        border = new Border(tag);

        boolean firstPart = true;
        for (XMLIterator i = new XMLIterator(tag); i.hasNext(); ) {
            Element subtag = i.next();
            if (subtag.getTagName().equals("part")) {
                Part p = new Part(this, subtag);
                if (firstPart) {
                    firstPart = false;
                    p.setStartNewPage(true);
                }
                parts.add(p);
            } else if (subtag.getTagName().equals("layout"))
                layout = LayoutFactory.createLayout(subtag);
            else if (subtag.getTagName().equals("paper"))
                pageFormat = new PageFormat(subtag);
            else if (subtag.getTagName().equals("border"))
                border = new Border(subtag);
            else if (subtag.getTagName().equals("section")) {
                Section s = new Section(this, subtag);
                String type = s.getType();
                if (type.equals("pageHeader"))
                    pageHeader = s;
                else if (type.equals("pageFooter"))
                    pageFooter = s;
                else
                    throw new RepException("Invalid section type in <report>: '" + 
                            type + "'");
            } else
                throw new RepException("Invalid part component");
        }
        
        if (layout == null)
            layout = new SimpleLayout();
        if (border == null)
            border = new Border();
        if (pageFormat == null)
            pageFormat = new PageFormat();
        
        Part prevPart = null;
        for (Iterator i = parts.listIterator(); i.hasNext(); ) {
            Part p = (Part)i.next();
            if ((prevPart != null) && p.getStartNewPage())
                p.setFinishOldPage(true);
            prevPart = p;
        }
        if (prevPart != null)
            prevPart.setFinishOldPage(true);
    }
    
    public GeneratedReport generate() throws RepException {
        return generate(new DataSource(), null, FRONT_SIDE | BACK_SIDE);
    }
    
    public GeneratedReport generate(DataSource dataSource) throws RepException {
        return generate(dataSource, null, FRONT_SIDE | BACK_SIDE);
    }
    
    public GeneratedReport generate(DataSource dataSource, Map parameters) throws RepException {
        return generate(dataSource, parameters, FRONT_SIDE | BACK_SIDE);
    }
    
    public GeneratedReport generate(DataSet dataSet) throws RepException {
        return generate(dataSet, null, FRONT_SIDE | BACK_SIDE);
    }
    
    public GeneratedReport generate(DataSet dataSet, Map parameters) throws RepException {
        return generate(dataSet, parameters, FRONT_SIDE | BACK_SIDE);
    }
    
    public GeneratedReport generate(DataSource dataSource, int side) throws RepException {
        return generate(dataSource, null, side);
    }
    
    public GeneratedReport generate(DataSet dataSet, Map parameters, int side) throws RepException {
        return generate(new DataSource(dataSet), parameters, side);
    }
    
    public GeneratedReport generate(DataSource dataSource, Map parameters, int side) throws RepException {
        if (((side & BACK_SIDE) != 0) && (! isTwoSided())) {
            side = FRONT_SIDE;
	}
        GenContext ctx = new GenContext(new GeneratedReport(name), dataSource, side, parameters, this);
        if (((side & BACK_SIDE) == 0) && ((side & FRONT_SIDE) == 0))
            return ctx.getGenReport();
        
        ctx.pushPageHeader(pageHeader);
        ctx.pushPageFooter(pageFooter);
        ctx.pushLayout(layout);
        ctx.pushBorder(border);
        ctx.pushPageFormat(pageFormat);
        
        for (Iterator i = parts.listIterator(); i.hasNext(); ) {
            Part part = (Part)i.next();
            part.generate(ctx);
        }

        ctx.popPageHeader(pageHeader);
        ctx.popPageFooter(pageFooter);
        ctx.popLayout(layout);
        ctx.popBorder(border);
        ctx.popPageFormat(pageFormat);
        
        return ctx.getGenReport();
    }
    
    public GeneratedReport generate(DataSet dataSet, int side) throws RepException {
        return generate(new DataSource(dataSet), null, side);
    }
    
    public PageFormat getPageFormat() {
        return pageFormat;
    }
    
    public void setPageFormat(PageFormat pf) {
        pageFormat = pf;
    }
    
    public Border getBorder() {
        return border;
    }
        
    public boolean isTwoSided() {
        for (Iterator i = parts.listIterator(); i.hasNext(); ) {
            Part part = (Part)i.next();
            if (part.isTwoSided())
                return true;
        }
        return false;
    }
    
    public DataLoader getDataLoader() {
        return dataLoader;
    }
    
    public void setDataLoader(DataLoader dataLoader) {
        this.dataLoader = dataLoader;
    }
    
    public void addNamedSection(String name, Section section) {
        namedSections.put(name, section);
    }
    
    public Section getNamedSection(String name) {
        return (Section)namedSections.get(name);
    }
    
    public Enumeration getParts() {
        return new EnumIteratorAdapter(parts.iterator());
    }
    
    public Section getPageHeader() {
        return pageHeader;
    }
    
    public Section getPageFooter() {
        return pageFooter;
    }
}
