/*
 * ReportView.java
 *
 * Created on 2  2001 ., 17:30
 */

package com.fn.repway;


import java.util.*;
import java.awt.image.VolatileImage;



/**
 *
 * @author  asso
 * @version
 */
public class ReportView extends javax.swing.JComponent implements javax.swing.Scrollable {
    private static final int bordersX = 10;
    private static final int bordersY = 10;
    private static final int shadowWidth = 10;
    private static final int shadowHeight = 10;
    private int pageNo = 0;
    private GeneratedReport report = null;
    private double zoom = 1.0;
    private static final double ppmm = 2.45;
    private List pageListeners = new LinkedList();
    private List zoomListeners = new LinkedList();
    private VolatileImage drawBuf = null;
    
    /** Creates new ReportView */
    public ReportView() {
        super();
    }
    
    public ReportView(GeneratedReport report) {
        super();
        setGeneratedReport(report);
    }
    
    public boolean getScrollableTracksViewportWidth() {
        return true;
    }
    
    public int getScrollableBlockIncrement(java.awt.Rectangle rectangle, int param, int param2) {
        return 40;
    }
    
    public boolean getScrollableTracksViewportHeight() {
        return true;
    }
    
    public java.awt.Dimension getPreferredScrollableViewportSize() {
        return getPreferredSize();
    }
    
    public int getScrollableUnitIncrement(java.awt.Rectangle rectangle, int param, int param2) {
        return 10;
    }
    
    public void addPageListener(ChangePageListener listener) {
        pageListeners.add(listener);
        if ((report != null) && (pageNo >= 0) && (pageNo < report.getNumberOfPages()))
            listener.onPageSelect(pageNo);
    }
    
    public void removePageListener(ChangePageListener listener) {
        pageListeners.remove(listener);
    }
    
    protected void fireChangePage() {
        for (Iterator i = pageListeners.listIterator(); i.hasNext(); ) {
            ChangePageListener pl = (ChangePageListener)i.next();
            pl.onPageSelect(pageNo);
        }
    }
    
    public void addZoomListener(ChangeZoomListener listener) {
        zoomListeners.add(listener);
        listener.onZoomChange(zoom);
    }
    
    public void removeZoomListener(ChangeZoomListener listener) {
        zoomListeners.remove(listener);
    }
    
    protected void fireChangeZoom() {
        for (Iterator i = zoomListeners.listIterator(); i.hasNext(); ) {
            ChangeZoomListener zl = (ChangeZoomListener)i.next();
            zl.onZoomChange(zoom);
        }
    }
    
    public void setPageNo(int no) throws Exception {
        if ((report == null) || (no < 0) || (no >= report.getNumberOfPages()))
            throw new Exception("Invalid page index");
        pageNo = no;
        fireChangePage();
        updateView();
    }
    
    public int getPageNo() {
        return pageNo;
    }
    
    private java.awt.Dimension getMinPageSize(java.awt.Dimension rv) {
        rv.width = 100;
        rv.height = 100;
        if ((report != null) && (pageNo >= 0) && (pageNo < report.getNumberOfPages())) {
            Page page = report.getPage(pageNo);
            int w = (int)(page.getPageFormat().getWidth() * zoom * ppmm) + 20;
            int h = (int)(page.getPageFormat().getHeight() * zoom * ppmm) + 20;
            if (w < 100) w = 100;
            if (h < 100) h = 100;
            if (page.getPageFormat().getOrient() != PageFormat.PORTRAIT) {
                int a = w;
                w = h;
                h = a;
            }
            rv.width = w;
            rv.height = h;
        }
        return rv;
    }
    
    public java.awt.Dimension getMinPageSize() {
        java.awt.Dimension d = new java.awt.Dimension();
        return getMinPageSize(d);
    }
    
    private void calcSize() {
        if ((report != null) && (pageNo >= 0) && (pageNo < report.getNumberOfPages())) {
            java.awt.Dimension d = getSize();
            java.awt.Dimension md = getMinPageSize();
            int w = md.width > d.width ? md.width : d.width;
            int h = md.height > d.height ? md.height : d.height;
            setSize(w, h);
        }
    }
    
 /*   public java.awt.Dimension getMinimumSize() {
        return getMinPageSize();
    } */
    
 /*   public java.awt.Dimension getMinimumSize(java.awt.Dimension rv) {
        return getMinPageSize(rv);
    }     */
    
    public void setSize(int width, int height) {
        java.awt.Dimension md = getMinPageSize();
        int w = md.width > width ? md.width : width;
        int h = md.height > height ? md.height : height;
        super.setSize(w, h);
    }
    
    public void setSize(java.awt.Dimension d) {
        setSize(d.width, d.height);
    }
    
    private void renderOffscreen(Page page, int w, int h, double pW, double pH) {
        do {
            if (drawBuf.validate(getGraphicsConfiguration()) ==
		VolatileImage.IMAGE_INCOMPATIBLE)
	    {
		// old vImg doesn't work with new GraphicsConfig; re-create it
		drawBuf = createVolatileImage(w, h);
	    }
            java.awt.Graphics2D g = drawBuf.createGraphics();

            java.awt.geom.AffineTransform oldt = g.getTransform();
            java.awt.geom.AffineTransform newt = new java.awt.geom.AffineTransform(oldt);
            newt.scale(zoom*ppmm, zoom*ppmm);
            g.setTransform(newt);
            g.setPaint(java.awt.Color.white);
            g.fill(new java.awt.geom.Rectangle2D.Double(0.0, 0.0, pW, pH));
            g.setPaint(java.awt.Color.black);
            g.setRenderingHint(java.awt.RenderingHints.KEY_ANTIALIASING, 
                    java.awt.RenderingHints.VALUE_ANTIALIAS_ON);
            g.setRenderingHint(java.awt.RenderingHints.KEY_TEXT_ANTIALIASING, 
                    java.awt.RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
            page.draw(g);
            g.setTransform(oldt);
            g.dispose();
        } while (drawBuf.contentsLost());
    }
    
    public void paintComponent(java.awt.Graphics g) {
        java.awt.Graphics2D g2 = (java.awt.Graphics2D)g;
        java.awt.Dimension d = getSize();
        
        g2.setPaint(getBackground());
        g2.fill(new java.awt.Rectangle(0, 0, d.width, d.height));
        
        if ((report != null) && (pageNo >= 0) && (pageNo < report.getNumberOfPages())) {
            Page page = report.getPage(pageNo);
            double pW = page.getPageFormat().getWidth();
            double pH = page.getPageFormat().getHeight();
            int x, y, w, h;
            
            if (page.getPageFormat().getOrient() == PageFormat.PORTRAIT) {
                x = (int)((d.width - (pW * zoom * ppmm)) / 2);
                y = (int)((d.height - (pH * zoom * ppmm)) / 2);
                w = (int)(pW * zoom * ppmm);
                h = (int)(pH * zoom * ppmm);
            }
            else {
                x = (int)((d.width - (pH * zoom * ppmm)) / 2);
                y = (int)((d.height - (pW * zoom * ppmm)) / 2);
                w = (int)(pH * zoom * ppmm);
                h = (int)(pW * zoom * ppmm);
            }
            
            if (drawBuf == null) {
                drawBuf = createVolatileImage(w, h);
                renderOffscreen(page, w, h, pW, pH);
            }
            
            do {
                int returnCode = drawBuf.validate(getGraphicsConfiguration());
                if (returnCode == VolatileImage.IMAGE_RESTORED) {
                    // Contents need to be restored
                    renderOffscreen(page, w, h, pW, pH);
                } else if (returnCode == VolatileImage.IMAGE_INCOMPATIBLE) {
                    // old vImg doesn't work with new GraphicsConfig; re-create it
                    drawBuf = createVolatileImage(w, h);
                    renderOffscreen(page, w, h, pW, pH);
                }
                
                g2.drawImage(drawBuf, x, y, this);
            } while (drawBuf.contentsLost());
            
            java.awt.geom.AffineTransform oldt = g2.getTransform();
            java.awt.geom.AffineTransform newt = new java.awt.geom.AffineTransform(oldt);
            newt.translate(x, y);
            newt.scale(zoom*ppmm, zoom*ppmm);
            g2.setTransform(newt);
            g2.setPaint(java.awt.Color.black);
            java.awt.Stroke oldStroke = g2.getStroke();
            g2.setStroke(new java.awt.BasicStroke(0.5f));
            g2.draw(new java.awt.geom.Rectangle2D.Double(0.0, 0.0, pW, pH));
            g2.setStroke(oldStroke);
            g2.fill(new java.awt.geom.Rectangle2D.Double(pW, 2.0, 2.0, pH));
            g2.fill(new java.awt.geom.Rectangle2D.Double(2.0, pH, pW, 2.0));
        }
    }
        
    public void updateView() {
        calcSize();
        drawBuf = null;
        revalidate();
        repaint();
    }
    
    /** Getter for property generatedReport.
     * @return Value of property generatedReport.
     */
    public GeneratedReport getGeneratedReport() {
        return report;
    }
    
    /** Setter for property generatedReport.
     * @param generatedReport New value of property generatedReport.
     */
    public void setGeneratedReport(GeneratedReport generatedReport) {
        report = generatedReport;
        try {
            setPageNo(0);
        } catch (Exception e) { }
    }
    
    public void setZoom(double zoom) {
        this.zoom = zoom;
        if (this.zoom < 0.1) this.zoom = 0.1;
        if (this.zoom > 6.0) this.zoom = 6.0;
        fireChangeZoom();
        updateView();
    }
    
    public double getZoom() {
        return zoom;
    }
    
    public void fitWidth(java.awt.Dimension size) {
        double z;

        if ((report == null) || (pageNo < 0) || (pageNo >= report.getNumberOfPages()))
            return;

        Page page = report.getPage(pageNo);
        if (page.getPageFormat().getOrient() == PageFormat.PORTRAIT)
            z = (size.width - 20) / (page.getPageFormat().getWidth() * ppmm);
        else
            z = (size.width - 20) / (page.getPageFormat().getHeight() * ppmm);

        setZoom(z);
    }
    
    public void fitPage(java.awt.Dimension size) {
        double zoom_w, zoom_h, z;
        
        if ((report == null) || (pageNo < 0) || (pageNo >= report.getNumberOfPages()))
            return;
        
        Page page = report.getPage(pageNo);
        if (page.getPageFormat().getOrient() == PageFormat.PORTRAIT) {
            zoom_w = (size.width - 20) / (page.getPageFormat().getWidth() * ppmm);
            zoom_h = (size.height - 20) / (page.getPageFormat().getHeight() * ppmm);
        } else {
            zoom_w = (size.width - 20) / (page.getPageFormat().getHeight() * ppmm);
            zoom_h = (size.height - 20) / (page.getPageFormat().getWidth() * ppmm);
        }

        if (zoom_w < zoom_h)
            z = zoom_w;
        else
            z = zoom_h;

        setZoom(z);
    }
}
