/*
 * Section.java
 *
 * Created on 9  2001 ., 12:21
 */

package com.fn.repway;

import java.util.*;
import org.w3c.dom.Element;

/**
 *
 * @author  asso
 * @version 
 */
class Section {
    
    private double width = -1.0;
    private double height = -1.0;
    
    private Section header = null;
    private Section footer = null;
    private Section pageBreak = null;
    private Section pageStart = null;
    private Section columnBreak = null;
    private Section columnStart = null;
    private List nested = new LinkedList();
    private String type;
    private String name;
    private String dataSet = null;
    private Report report;
    //private boolean dontMove = false;
    private String grouping;
    
    private List frontLayers = new LinkedList();
    private List backLayers = new LinkedList();

    /** Creates new Section */
    Section(Report report, Element tag) throws RepException {        
        this.report = report;
        type = XMLUtils.getAttrib(tag, "type", "body");
        name = XMLUtils.getAttrib(tag, "name", "");
        if (name.length() > 0)
            report.addNamedSection(name, this);
        dataSet = XMLUtils.getAttrib(tag, "dataset", "");
        height = XMLUtils.getAttrib(tag, "height", 0.0);
        width = XMLUtils.getAttrib(tag, "width", 0.0);
        grouping = XMLUtils.getAttrib(tag, "grouping", "");
       // dontMove = XMLUtils.getAttrib(tag, "dontMove", false);
        for (XMLIterator i = new XMLIterator(tag); i.hasNext(); ) {
            Element subtag = i.next();
            String stn = subtag.getTagName();
            if (stn.equals("section")) {
                Section s = new Section(report, subtag);
                String t = s.getType();
                if (t.equals("header"))
                    header = s;
                else if (t.equals("footer"))
                    footer = s;
                else if (t.equals("pagebreak"))
                    pageBreak = s;
                else if (t.equals("pagestart"))
                    pageStart = s;
                else if (t.equals("columnbreak"))
                    columnBreak = s;
                else if (t.equals("columnstart"))
                    columnStart = s;
                else if (t.equals("nested"))
                    nested.add(s);
                else
                    throw new RepException("Invalid section type '" + t + "'");
            } else if (stn.equals("face"))
                addLayers(frontLayers, subtag);
            else if (stn.equals("back"))
                addLayers(backLayers, subtag);
            else
                throw new RepException("Invalid section component '" + stn + "'");
        }
    }

    private boolean hasGrouping(String dataSetName, boolean ignoreSelf) {
        if (! ignoreSelf)
            if ((grouping.length() > 0) && (dataSet != null) && (dataSet.equals(dataSetName)))
                return true;
        
        if ((header != null) && header.hasGrouping(dataSetName, false)) return true;
        if ((footer != null) && footer.hasGrouping(dataSetName, false)) return true;
        if ((pageBreak != null) && pageBreak.hasGrouping(dataSetName, false)) return true;
        if ((pageStart != null) && pageStart.hasGrouping(dataSetName, false)) return true;
        if ((columnBreak != null) && columnBreak.hasGrouping(dataSetName, false)) return true;
        if ((columnStart != null) && columnStart.hasGrouping(dataSetName, false)) return true;
        
        for (Iterator i = nested.listIterator(); i.hasNext(); ) {
            Section s = (Section)i.next();
            if (s.hasGrouping(dataSetName, false))
                return true;
        }
        return false;
    }

    private void addLayers(List layers, Element tag) throws RepException {
        for (XMLIterator i = new XMLIterator(tag); i.hasNext(); ) {
            Element subtag = i.next();
            String n = subtag.getTagName();
            if (n.equals("layer"))
                layers.add(new Layer(report, subtag));
            else
                throw new RepException("Layer expected but '" + n + "' found.");
        }
    }

    double getHeight() {
        return height;
    }

    double getWidth() {
        return width;
    }

    String getType() {
        return type;
    }
    
    String getName() {
        return name;
    }

    public boolean isTwoSided() {
        if ((backLayers != null) && (backLayers.size() > 0))
            return true;
        if ((header != null) && header.isTwoSided()) return true;
        if ((footer != null) && footer.isTwoSided()) return true;
        if ((pageBreak != null) && pageBreak.isTwoSided()) return true;
        if ((pageStart != null) && pageStart.isTwoSided()) return true;
        if ((columnBreak != null) && columnBreak.isTwoSided()) return true;
        if ((columnStart != null) && columnStart.isTwoSided()) return true;
        
        for (Iterator i = nested.listIterator(); i.hasNext(); ) {
            Section s = (Section)i.next();
            if (s.isTwoSided())
                return true;
        }
        
        return false;
    }
    
    public boolean checkPredicate(GenContext ctx) {
        return true;
    }
    
    public Band generateBody(GenContext ctx) throws RepException {
        Band band = new Band(width, height);
        
        if ((ctx.getSide() & Report.FRONT_SIDE) != 0)
            for (Iterator i = frontLayers.listIterator(); i.hasNext(); )
                ((Layer)i.next()).generate(ctx, band, true);
        
        if ((ctx.getSide() & Report.BACK_SIDE) != 0)
            for (Iterator i = backLayers.listIterator(); i.hasNext(); )
                ((Layer)i.next()).generate(ctx, band, false);
        
        return band;
    }
    
    private void generateNested(GenContext ctx) throws RepException {
        for (Iterator i = nested.listIterator(); i.hasNext(); ) {
            Section s = (Section)i.next();
            if (s.checkPredicate(ctx))
                s.generate(ctx);
        }
    }
    
    public void generate(GenContext ctx) throws RepException {
        DataSet ds = null;
        if ((dataSet != null) && (! dataSet.equals(""))) {
            ds = ctx.getDataSet(dataSet);
            if ((ds == null) || (! ds.hasCurrent()))
                return;
        }
        
        ctx.pushPageBreak(pageBreak);
        ctx.pushPageStart(pageStart);
        ctx.pushColumnBreak(columnBreak);
        ctx.pushColumnStart(columnStart);
        
        if (header != null)
            header.generate(ctx);
        
        Layout layout = ctx.getLayout();
        boolean hasGrouping = (ds != null) && hasGrouping(dataSet, true);
        boolean exitNow = true;
        Value lastGroupingValue = null;
        if ((ds != null) && (grouping != null) && (grouping.length() > 0))
            lastGroupingValue = ds.getValue(grouping);
        do {
            layout.addBand(ctx, generateBody(ctx));
            generateNested(ctx);
            if (ds != null) {
                exitNow = hasGrouping ? (! ds.hasCurrent()) : (! ds.gotoNext());
                if ((! exitNow) && (grouping != null) && (grouping.length() > 0)) {
                    Value v = ds.getValue(grouping);
                    if (! v.equals(lastGroupingValue))
                        exitNow = true;
                    lastGroupingValue = v;
                }
            }
        } while (! exitNow);
        
        if (footer != null)
            footer.generate(ctx);
        
        ctx.popPageBreak(pageBreak);
        ctx.popPageStart(pageStart);
        ctx.popColumnBreak(columnBreak);
        ctx.popColumnStart(columnStart);
    }
    
    public List getFrontLayers() {
        return frontLayers;
    }
    
    public List getBackLayers() {
        return backLayers;
    }
}
