/*
 * ThreeDAxisDrawer.java
 *
 * Created on 26  2003 ., 14:29
 */

package com.fn.repway;


import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 */
public class ThreeDHistDrawer implements HistDrawer {
    
    private double axisThickness;
    private Color axisColor;
    private double axisTickThickness;
    private Color axisTickColor;
    private double axisTickSize;
    private Label tickLabel;
    private double tickLabelGap;
    private double deepX;
    private double deepY;
    private Color axisFillColor;
    
    /** Creates a new instance of ThreeDAxisDrawer */
    public ThreeDHistDrawer(Element tag) throws RepException {
        Element subtag = XMLUtils.getSubTag(tag, "style");
        axisThickness = XMLUtils.getAttrib(subtag, "axisThickness", 0.5);
        axisColor = XMLUtils.getColorAttrib(subtag, "axisColor", "#000000");
        axisTickThickness = XMLUtils.getAttrib(subtag, "tickThickness", 0.4);
        axisTickSize = XMLUtils.getAttrib(subtag, "tickSize", 2.0);
        axisTickColor = XMLUtils.getColorAttrib(subtag, "tickColor", "#000000");
        tickLabelGap = XMLUtils.getAttrib(subtag, "tickLabelGap", 1.0);
        axisFillColor = XMLUtils.getColorAttrib(subtag, "axisFillColor", "#DEDEDE");
        deepX = XMLUtils.getAttrib(subtag, "deepX", 3.0);
        deepY = XMLUtils.getAttrib(subtag, "deepY", 3.0);
        try {
            Element labelTag = XMLUtils.getSubTag(subtag, "tickLabel");
            if (labelTag != null)
                tickLabel = new Label(labelTag);
        } catch (Exception e) { }
    }
    
    public void drawAxis(GenContext ctx, Band band, boolean front, Axis axis, 
            Rectangle2D.Double area, double koof) 
    {
        band.add(front, makeTrap(area.x, area.y + area.height, 
            area.x + area.width - deepX, area.y + area.height, 
            area.x + area.width, area.y + area.height - deepY, 
            area.x + deepX, area.y + area.height - deepY, 
            axisThickness, axisColor, axisFillColor));
        band.add(front, makeTrap(area.x, area.y + area.height, 
            area.x + deepX, area.y + area.height - deepY, 
            area.x + deepX, area.y, 
            area.x, area.y + deepY, 
            axisThickness, axisColor, axisFillColor));
        band.add(front, new CompBar(area.x + deepX, area.y, area.width - deepX, 
                area.height - deepX, axisThickness, axisColor, axisFillColor));
        
        koof = koof * ((area.height - deepY) / area.height);
        double tickStep = axis.getTickInterval();
        if (tickStep > 0.0) {
            double max = axis.getMaxValue();
            for (double i = 0.0; i <= max; i += tickStep) {
                double h = area.y + area.height - i*koof;
                band.add(front, new CompLine(area.x, h, -axisTickSize, 0.0, 
                        axisTickThickness, axisTickColor));
                band.add(front, new CompLine(area.x, h, deepX, -deepY, 
                        axisTickThickness, axisTickColor));
                band.add(front, new CompLine(area.x + deepX, h - deepY, 
                        area.width - deepX, 0.0, axisTickThickness, axisTickColor));
                if (tickLabel != null) {
                    ctx.setVariable("__axisTick", new Value(i));
                    tickLabel.setLeft(area.x - axisTickSize - tickLabel.getWidth() - tickLabelGap);
                    tickLabel.setTop(h - tickLabel.getHeight() / 2.0);
                    try { tickLabel.generate(ctx, band, front); } catch (Exception e) { }
                }
            }
        }
    }
    
    private CompPoly makeTrap(double x1, double y1, double x2, double y2, 
            double x3, double y3, double x4, double y4, double thickness, 
            Color border, Color fill) 
    {
        double xCoords[] = new double[4];
        double yCoords[] = new double[4];
        xCoords[0] = x1;  yCoords[0] = y1;
        xCoords[1] = x2;  yCoords[1] = y2;
        xCoords[2] = x3;  yCoords[2] = y3;
        xCoords[3] = x4;  yCoords[3] = y4;
        return new CompPoly(xCoords, yCoords, thickness, border, fill);
    }
    
    public void drawBar(GenContext ctx, Band band, boolean front, Axis axisY, 
            Rectangle2D.Double area, double position, double width, double value, 
            double thickness, Color border, Color fill) 
    {
        value = value * ((area.height - deepY) / area.height);
        double left = area.x+position;
        double top = area.y + area.height - value;
        double right = left + width;
        double bottom = top + value;
        
        band.add(front, makeTrap(left, top, 
            right-deepX, top, 
            right-deepX, bottom, 
            left, bottom, 
            thickness, border, fill));
        band.add(front, makeTrap(left, top, 
                right - deepX, top,
                right, top - deepY,
                left + deepX, top - deepY,
                thickness, border, fill));
        band.add(front, makeTrap(right - deepX, bottom, 
                right, bottom - deepY,
                right, top - deepY,
                right - deepX, top,
                thickness, border, fill));
    }
    
}
