/*
 * URLDataLoader.java
 *
 * Created on 11  2001 ., 12:45
 */

package com.fn.repway;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 *
 * @author  asso
 */
public class URLDataLoader implements DataLoader {

    private URL prefix;
    
    /** Creates a new instance of URLDataLoader */
    public URLDataLoader() {
        prefix = null;
    }
    
    public URLDataLoader(String prefix) throws RepException {
        if (prefix == null)
            this.prefix = null;
        else
        {
            try {
		if (! prefix.endsWith("/"))
		    this.prefix = new URL(prefix + "/");
		else
		    this.prefix = new URL(prefix);
            } catch (Exception e) {
                throw new RepException("Invalid URL prefix '" + prefix + "'");
            }
        }
    }

    private static final int BLOCK_SIZE = 10240;
    
    public byte[] load(String name) throws RepException {
        byte[] data = null;
	String urlstr = "";
        try {
            URL url;
            if (prefix != null)
                url = new URL(prefix, name);
            else
                url = new URL(name);
	    urlstr = url.toString();
            InputStream is = url.openStream();
            int size = 0;
            List blocks = new LinkedList();
            int downloaded = 0;
            do {
                byte[] block = new byte[BLOCK_SIZE];
                downloaded = is.read(block);
                size += downloaded;
                if (size > 0x0FFFFFFF)
                    throw new RepException("Data is too big");
                if (downloaded > 0)
                    blocks.add(block);
            } while (downloaded == BLOCK_SIZE);
            is.close();
            data = new byte[size + 10];
	    if (data == null)
		throw new RepException("Error allocating data");
            int pos = 0;
            for (Iterator i = blocks.listIterator(); i.hasNext(); ) {
                byte[] block = (byte[])i.next();
		if (block == null)
		    throw new RepException("Block is null");
                int len = block.length;
		if (pos + len > size)
		    len = size - pos;
                System.arraycopy(block, 0, data, pos, len);
                pos += len;
            }
        } catch (Exception e) {
            throw new RepException("Error loading URL '" + urlstr + "'+'" + name + "': " + e.getMessage());
        }
        return data;
    }
    
}
