/*
 * Value.java
 *
 * Created on 27  2003 ., 10:59
 */

package com.fn.repway;


import java.text.DateFormat;
import java.text.NumberFormat;
import java.util.Date;


/**
 *
 * @author  asso
 */
public class Value {

    public static final int INTEGER = 1;
    public static final int DOUBLE = 2;
    public static final int STRING = 3;
    public static final int DATASET = 4;
    public static final int DATE = 5;
    public static final int TIMEINT = 6;
    public static final int TIME = 7;
    public static final int DATETIME = 8;

    private int type;
    private int intValue;
    private double doubleValue;
    private String stringValue;
    private DataSet dataSetValue;
    private Date dateValue;
    
    public Value(int v) {
        type = INTEGER;
        intValue = v;
    }
    
    public Value(double v) {
        type = DOUBLE;
        doubleValue = v;
    }
    
    public Value(String v) {
        type = STRING;
        stringValue = v;
    }
    
    public Value(DataSet v) {
        type = DATASET;
        dataSetValue = v;
    }

    public Value(Date v) {
        type = DATE;
        dateValue = v;
    }

    public Value(int type, Date v) {
        this.type = type;
        dateValue = v;
    }

    public Value(int type, int v) {
        this.type = type;
        intValue = v;
    }

    public int getType() {
        return type;
    }

    public int getAsInt() throws RepException {
        switch (type) {
            case TIMEINT:
            case INTEGER: return intValue;
            case DOUBLE: return (int)doubleValue;
            case STRING:
                try {
                    return Integer.parseInt(stringValue);
                } catch (Exception e) {
                    throw new RepException("Can't cast String to int");
                }                
            case TIME:
            case DATETIME:
            case DATE: throw new RepException("Can't cast Date to int");
            case DATASET: throw new RepException("Can't cast DataSet to int");
        }
        throw new RepException("Unknown data type");
    }

    public double getAsDouble() throws RepException {
        switch (type) {
            case TIMEINT:
            case INTEGER: return intValue;
            case DOUBLE: return doubleValue;
            case STRING:
                if ((stringValue == null) || (stringValue.length() == 0))
                    return 0.0;
                try {
                    return Double.parseDouble(stringValue);
                } catch (Exception e) {
                    throw new RepException("Can't cast String '" + stringValue + "' to double");
                }
            case TIME:
            case DATETIME:
            case DATE: throw new RepException("Can't cast Date to double");
            case DATASET: throw new RepException("Can't cast DataSet to double");
        }
        throw new RepException("Unknown data type");
    }

    private String intToTimeInt(int v) {
        int m = v / 60;
        int s = v - m * 60;
        int h = m / 60;
        m = m - h * 60;
        StringBuffer sb = new StringBuffer();
        NumberFormat nf = NumberFormat.getIntegerInstance();
        nf.setMinimumIntegerDigits(2);
        sb.append(nf.format(h)).append(":").append(nf.format(m))
            .append(":").append(nf.format(s));
        return sb.toString();
    }

    public String getAsString() throws RepException {
        switch (type) {
            case INTEGER: return Integer.toString(intValue);
            case DOUBLE: return NumberFormat.getNumberInstance().format(doubleValue);
            case STRING: return stringValue;
            case DATASET: throw new RepException("Can't cast DataSet to String");
            case TIMEINT: return intToTimeInt(intValue);
            case DATE: return DateFormat.getDateInstance(DateFormat.SHORT).format(dateValue);
            case TIME: return DateFormat.getTimeInstance(DateFormat.SHORT).format(dateValue);
            case DATETIME: return DateFormat.getDateTimeInstance(DateFormat.SHORT, 
                    DateFormat.SHORT).format(dateValue);
        }
        throw new RepException("Unknown data type");
    }

    public DataSet getAsDataSet() throws RepException {
        if (type == DATASET)
            return dataSetValue;
        else
            throw new RepException("Can't cast anything to DataSet");
    }

    public Date getAsDate() throws RepException {
        if (type == DATE)
            return dateValue;
        else
            throw new RepException("Can't cast anything to Date");
    }

    public int getAsTimeInt() throws RepException {
        return getAsInt();
    }
    
    public boolean equals(Object obj) {
        Value v = (Value)obj;
        try {
            switch (type) {
                case TIMEINT:
                case INTEGER: return intValue == v.getAsInt();
                case DOUBLE: return doubleValue == v.getAsDouble();
                case STRING: return stringValue.equals(v.getAsString());
                case TIME:
                case DATETIME:
                case DATE: return dateValue.equals(v.getAsDate());
                case DATASET: throw new RepException("Can't compare DataSet values");
            }
        } catch (Exception e) { }
        return false;
    }
}
