package com.fn.repway;


import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import org.w3c.dom.Element;


public class VarOp extends RepElement implements DataField {

    private String variable;
    private int action;
    private List args;

    private static final int ACTION_ADD = 1;
    private static final int ACTION_SUB = 2;
    private static final int ACTION_DIV = 3;
    private static final int ACTION_MUL = 4;

    public VarOp(Element tag) throws RepException {
        super(tag);

        variable = XMLUtils.getAttrib(tag, "res", "");

        String s = XMLUtils.getAttrib(tag, "action", "+");
        if (s.equals("+"))
            action = ACTION_ADD;
        else if (s.equals("-"))
            action = ACTION_SUB;
        else if (s.equals("/"))
            action = ACTION_DIV;
        else if (s.equals("*"))
            action = ACTION_MUL;
        else
            throw new RepException("Invalid variable action '" + s + "'");

        args = new LinkedList();
        for (XMLIterator i = new XMLIterator(tag); i.hasNext(); ) {
            Element subtag = i.next();
            args.add(FieldFactory.createField(subtag.getTagName(), subtag));
        }
        
        if (args.size() < 1)
            throw new RepException("Missing varop arguments");
    }
    
    public void generate(GenContext ctx, Band band, boolean front) throws RepException {
        getText(ctx);
    }

    private Value doAction(Value a1, Value a2) throws RepException {
        switch (action) {
            case ACTION_ADD: return new Value(a1.getAsDouble() + a2.getAsDouble());
            case ACTION_SUB: return new Value(a1.getAsDouble() - a2.getAsDouble());
            case ACTION_DIV: return new Value(a1.getAsDouble() / a2.getAsDouble());
            case ACTION_MUL: return new Value(a1.getAsDouble() * a2.getAsDouble());
        }
        throw new RepException("Invalid varop");
    }
    
    public Value getText(GenContext ctx) throws RepException {
        Value res = null;
        for (Iterator i = args.iterator(); i.hasNext(); ) {
            Value v = ((DataField)i.next()).getText(ctx);
            if (res == null)
                res = v;
            else
                res = doAction(res, v);
        }
        if ((variable != null) && (variable.length() > 0))
            ctx.setVariable(variable, res);
        return res;
    }
    
}

