/*
 * MainWindow.java
 *
 * Created on 14  2003 ., 12:38
 */

package com.fn.repway;


import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import org.w3c.dom.Element;

import com.fn.repway.GeneratedReport;
import com.fn.repway.Report;
import com.fn.repway.XMLDataSet;
import com.fn.repway.XMLIterator;
import com.fn.repway.XMLUtils;
import com.fn.repway.Value;


/**
 *
 * @author  asso
 */
public class ViewerWindow extends javax.swing.JFrame {
    
    private boolean exitOnClose;
    
    /** Creates new form MainWindow */
    public ViewerWindow(boolean exitOnClose) {
        this.exitOnClose = exitOnClose;
        initComponents();
        SwingUtils.centerWindow(this);
    }
    
    public ViewerWindow(GeneratedReport rep) {
        exitOnClose = false;
        initComponents();
        reportPreview.setGeneratedReport(rep);
        SwingUtils.centerWindow(this);
    }

    public ViewerWindow(File file) {
        exitOnClose = false;
        initComponents();
        openReport(file);
        SwingUtils.centerWindow(this);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        reportPreview = new com.fn.repway.Preview();
        mainMenuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        fileOpenItem = new javax.swing.JMenuItem();
        fileSaveItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        fileExitItem = new javax.swing.JMenuItem();

        setTitle(I18n.getMsg("main.title"));
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        getContentPane().add(reportPreview, java.awt.BorderLayout.CENTER);

        fileMenu.setText(I18n.getMsg("main.menu.file"));
        fileOpenItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        fileOpenItem.setText(I18n.getMsg("main.menu.file.open"));
        fileOpenItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fileOpenItemActionPerformed(evt);
            }
        });

        fileMenu.add(fileOpenItem);

        fileSaveItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        fileSaveItem.setText(I18n.getMsg("main.menu.file.save"));
        fileSaveItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fileSaveItemActionPerformed(evt);
            }
        });

        fileMenu.add(fileSaveItem);

        fileMenu.add(jSeparator1);

        fileExitItem.setText(I18n.getMsg("main.menu.file.exit"));
        fileExitItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fileExitItemActionPerformed(evt);
            }
        });

        fileMenu.add(fileExitItem);

        mainMenuBar.add(fileMenu);

        setJMenuBar(mainMenuBar);

        pack();
    }//GEN-END:initComponents

    private void fileSaveItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileSaveItemActionPerformed
        GeneratedReport genRep = reportPreview.getGeneratedReport();
        if (genRep != null) {
            JFileChooser fc = new JFileChooser();
            int ret = fc.showSaveDialog(this);
            if (ret == JFileChooser.APPROVE_OPTION)
                saveReport(genRep, fc.getSelectedFile());
        }
    }//GEN-LAST:event_fileSaveItemActionPerformed

    private void saveReport(GeneratedReport genRep, File file) {
        try {
            PrintWriter out = new PrintWriter(new OutputStreamWriter(
                    new FileOutputStream(file), "utf-8"));
            genRep.save(out);
            out.close();
        } catch (Exception e) {
            e.printStackTrace();
            JOptionPane.showMessageDialog(this, I18n.m("error.fileSave"));
        }
    }

    private void openReport(File file) {
        try {
            GeneratedReport generated = new GeneratedReport(XMLUtils.parse(file));
            reportPreview.setGeneratedReport(generated);
        } catch (Exception e) {
            e.printStackTrace();
            JOptionPane.showMessageDialog(this, I18n.m("error.fileOpen"));
        }
    }

    private void fileOpenItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileOpenItemActionPerformed
        JFileChooser fc = new JFileChooser();
        int ret = fc.showOpenDialog(this);
        if (ret == JFileChooser.APPROVE_OPTION) {
            openReport(fc.getSelectedFile());
        }
    }//GEN-LAST:event_fileOpenItemActionPerformed

    private void fileExitItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fileExitItemActionPerformed
        exitForm(null);
    }//GEN-LAST:event_fileExitItemActionPerformed

    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        if (exitOnClose)
            System.exit(0);
        else {
            hide();
            dispose();
        }
    }//GEN-LAST:event_exitForm

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem fileExitItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JMenuItem fileOpenItem;
    private javax.swing.JMenuItem fileSaveItem;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JMenuBar mainMenuBar;
    private com.fn.repway.Preview reportPreview;
    // End of variables declaration//GEN-END:variables
    
}
