/*
 * XMLDataSet.java
 *
 * Created on 10  2001 ., 14:07
 */

package com.fn.repway;

import java.io.*;
import java.util.*;
import org.w3c.dom.Element;

/**
 *
 * @author  asso
 * @version 
 */
public class XMLDataSet implements DataSet {

    private String name;
    private Element tag;
    private XMLIterator iterator = null;
    private Map fields = null;             // TODO: optimize!
    private Map datasets = null;           // TODO: optimize!!!
    
    /** Creates new XMLDataSet */
    public XMLDataSet(Element tag) throws RepException {
        init(tag);
    }

    public XMLDataSet(Reader reader) throws RepException {
        Element tag;
        try {
            tag = XMLUtils.parse(reader);
        } catch (Exception e) {
	    throw new RepException("Error parsing XML DataSet: " + e.getMessage());
        }
        init(tag);
    }

    public XMLDataSet(String data) throws RepException {
        Element tag;
        try {
            tag = XMLUtils.parse(new StringReader(data));
        } catch (Exception e) {
	    throw new RepException("Error parsing XML DataSet: " + e.getMessage());
        }
        init(tag);
    }
    
    private void init(Element tag) throws RepException {
        this.tag = tag;
        if (tag == null) {
            iterator = null;
            return;
        }
        this.name = XMLUtils.getAttrib(tag, "name", "");
        iterator = new XMLIterator(tag);
        if ((iterator != null) && iterator.hasNext())
            processRecord(iterator.next());
    }

 /*   public boolean hasNext() {
        if (iterator == null)
            return false;
        return iterator.hasNext();
    }*/
    
    public boolean gotoNext() throws RepException {
        fields = null;
        datasets = null;
        if (iterator == null)
            return false;
        if (! iterator.hasNext())
            return false;
        processRecord(iterator.next());
        return true;
    }
    
    public String getName() {
        return name;
    }
    
    public Value getValue(String value) throws RepException {
        if (datasets != null) {
            XMLDataSet ds = (XMLDataSet)datasets.get(name);
            if (ds != null)
                return new Value(ds);
        }
        if (fields != null) {
            String s = (String)fields.get(name);
            if (s != null)
                return new Value(s);
        }
        throw new RepException("Field '" + value + "' not found");
    }
    
    // TODO: optimize!  VERY slow
    private void processRecord(Element tag) throws RepException {
        if (tag == null) return;
        if (! tag.getTagName().equals("record"))
            throw new RepException("<record> expected but '" + tag.getTagName() + "' found");
        fields = new HashMap();
        datasets = new HashMap();
        XMLIterator i = new XMLIterator(tag);
        if (i != null) {
            while (i.hasNext()) {
                Element subtag = i.next();
                if (subtag != null) {
                    String n = XMLUtils.getAttrib(subtag, "name", "");
                    if ((n == null) || (n.equals("")))
                        throw new RepException("Invalid field in '" + name + "' dataset");
                    String s = subtag.getTagName();
                    if (s.equals("field"))
                        fields.put(n, XMLUtils.getData(subtag));
                    else if (s.equals("dataset")) 
                        datasets.put(n, new XMLDataSet(subtag));
                    else
                        throw new RepException("Invalid record in '" + name + "' dataset");
                }
            }
        }
    }
    
    public boolean hasCurrent() {
        return ((fields != null) || (datasets != null));
    }
    
    public void reset() throws RepException {
        init(tag);
    }
    
}
