/*
 * XMLUtils.java
 *
 * Created on 25  2003 ., 11:38
 */

package com.fn.repway;


import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;


/**
 *
 * @author  asso
 */
public class XMLUtils {

    public static String getAttrib(Element tag, String attrib, String dflt) {
        if (tag == null) return dflt;
        String s = tag.getAttribute(attrib);
        return (s.length() < 1) ? dflt : s;
    }
    
    public static double getAttrib(Element tag, String attrib, double dflt) {
        if (tag == null) return dflt;
        String s = tag.getAttribute(attrib);
        if (s.length() < 1)
            return dflt;
        else {
            try {
                return Double.parseDouble(s);
            } catch (Exception e) {
                return dflt;
            }
        }
    }
    
    public static int getAttrib(Element tag, String attrib, int dflt) {
        if (tag == null) return dflt;
        String s = tag.getAttribute(attrib);
        if (s.length() < 1)
            return dflt;
        else {
            try {
                return Integer.parseInt(s);
            } catch (Exception e) {
                return dflt;
            }
        }
    }
    
    public static boolean getAttrib(Element tag, String attrib, boolean dflt) {
        if (tag == null) return dflt;
        String s = tag.getAttribute(attrib).toLowerCase();
        if (s.length() < 1) 
            return dflt;
        if (s.equals("on") || s.equals("yes") || s.equals("true") || s.equals("rulez"))
            return true;
        else if (s.equals("off") || s.equals("no") || s.equals("false") || s.equals("suxx"))
            return false;
        else
            return dflt;
    }
    
    public static Element parse(Reader reader) throws RepException {
        Document document = null;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        
        factory.setValidating(false);
        factory.setNamespaceAware(true);
        
        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            document = builder.parse(new InputSource(reader));
            Element root = document.getDocumentElement();
            if (root != null)
                return root;
            else
                throw new Exception("Root element not found");            
        } catch (SAXParseException spe) {
            // Error generated by the parser
            System.out.println("\n** Parsing error , line " + spe.getLineNumber()
            + ", uri " + spe.getSystemId());
            System.out.println(" " + spe.getMessage() );
            // Use the contained exception, if any
            Exception x = spe;
            if (spe.getException() != null)
                x = spe.getException();
            x.printStackTrace();
            throw new RepException("Parsing error, line " + spe.getLineNumber()
                    + ", uri " + spe.getSystemId() + " " + spe.getMessage());
        } catch (SAXException sxe) {
            // Error generated during parsing
            Exception x = sxe;
            if (sxe.getException() != null)
                x = sxe.getException();
            x.printStackTrace();
            throw new RepException(x.getMessage());
        } catch (ParserConfigurationException pce) {
            // Parser with specified options can't be built
            pce.printStackTrace();
            throw new RepException(pce.getMessage());
        } catch (IOException ioe) {
            // I/O error
            ioe.printStackTrace();
            throw new RepException(ioe.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RepException(e.getMessage());
        }
    }

    public static Element parse(File file) throws RepException {
        try {
            return parse(new FileReader(file));
        } catch (java.io.FileNotFoundException nfe) {
            throw new RepException("File '" + file.getPath() + "' not found");
        }
    }
    
    public static String getData(Element tag) {
        if (tag == null) return "";
        StringBuffer sb = new StringBuffer();
        NodeList nodeList = tag.getChildNodes();
        int len = nodeList.getLength();
        for (int i = 0; i < len; i++) {
            Node node = nodeList.item(i);
            short nodeType = node.getNodeType();
            if (nodeType == Node.TEXT_NODE)
                sb.append(node.getNodeValue());
        }
        return sb.toString();
    }
    
    public static Element getSubTag(Element element, String name) throws RepException {
        try {
            NodeList nodeList = element.getElementsByTagName(name);
            if (nodeList.getLength() == 0)
                return null;
            return (Element)nodeList.item(0);
        } catch (Exception e) {
            throw new RepException("subtag '" + name + "' not found");
        }
    }

    public static java.awt.Color getColorAttrib(Element tag, String attrib, String dfltClr) throws RepException {
        String clr;
        clr = getAttrib(tag, attrib, dfltClr);
        String s = clr.toLowerCase();
        java.awt.Color c;        
        try {
            c = java.awt.Color.decode(s);
        }
        catch (Exception e) {
            throw new RepException("Invalid color '" + s + "'.");
        }        
        return c;
    }

    public static Rectangle2D.Double getRectAttribs(Element tag, double x, 
            double y, double w, double h, double left, double top) 
    {
        if (tag == null)
            return new Rectangle2D.Double(left + x, top + y, w, h);
        return new Rectangle2D.Double(XMLUtils.getAttrib(tag, "left", x) + left, 
                XMLUtils.getAttrib(tag, "top", y) + top, 
                XMLUtils.getAttrib(tag, "width", w), XMLUtils.getAttrib(tag, "height", h));
    }

    public static String escape(String str) {
        if (str == null)
            return "";
        StringBuffer sb = new StringBuffer(str.length());
        int len = str.length();
        for (int i = 0; i < len; i++) {
            char ch = str.charAt(i);
            if (ch == '<')
                sb.append("&lt;");
            else if (ch == '>')
                sb.append("&gt;");
            else if (ch == '"')
                sb.append("&quot;");
            else if (ch == '&')
                sb.append("&amp;");
            else
                sb.append(ch);
        }
        return sb.toString();
    }
}
